@@ -24,6 +24,7 @@
 import json
 import os
 import pprint
+import shutil
 
 import converters
 
@@ -138,9 +139,16 @@ class ConfigViper(object):
     _sv_converters = {}
     """Converters dict."""
 
+    _sv_make_backups = True
+    """..."""
+
+    _sv_keep_backups = False
+    """..."""
+
 
     @staticmethod
-    def configure(pathname='', filename='', auto_save=True, separator='.'):
+    def configure(pathname='', filename='', separator='.',
+            auto_save=True, make_backups=True, keep_backups=False):
         """Set parameters that will be used by all instances of ``ConfigViper``.
         This method is intended to be called only once, when the host 
         application is preparing everything up to be ran.
@@ -152,18 +160,28 @@ class ConfigViper(object):
         :param filename: The configuration file name (usually) with ``.json``
             extension. If not set, defaults to ``configviper.json``.
 
+        :param separator: The config-path separator. Defaults to ``.``.
+
         :param auto_save: When a value is set (:meth:`~ConfigViper.set` method)
             or when :meth:`~ConfigViper.stabilize` method is called the
             configuration file will be automatically saved. 
             Defaults to ``True``.
 
-        :param separator: The config-path separator. Defaults to ``.``.
+        :param make_backups: Make a backup of configuration file right before
+            save operation. The backup filename should be created on the same
+            directory of original configuration file with the ``.backup``
+            appended. Defaults to ``True``.
+
+        :param keep_backups: Keep backup file after successful save.
+            Defaults to ``False``.
 
         """
         ConfigViper._sv_pathname = pathname
         ConfigViper._sv_filename = filename
-        ConfigViper._sv_save_on_set = bool(auto_save)
         ConfigViper._sv_path_separator = separator
+        ConfigViper._sv_save_on_set = bool(auto_save)
+        ConfigViper._sv_make_backups = make_backups
+        ConfigViper._sv_keep_backups = keep_backups
         ConfigViper._sv_converters = {}
         
         if not ConfigViper._sv_pathname:
@@ -184,9 +202,14 @@ class ConfigViper(object):
         self._pathname = ConfigViper._sv_pathname
         self._filename = ConfigViper._sv_filename
         self._confname = os.path.join(self._pathname, self._filename)
+        self._backupname = os.path.join(
+                self._pathname, '%s.backup' % self._filename)
         
         self._save_on_set = bool(ConfigViper._sv_save_on_set)
+        self._stop_saving__save_on_set_flag = self._save_on_set
         self._path_separator = ConfigViper._sv_path_separator
+        self._make_backups = ConfigViper._sv_make_backups
+        self._keep_backups = ConfigViper._sv_keep_backups
         
         # ensure path name exists
         if not os.path.exists(self._pathname):
@@ -288,14 +311,73 @@ class ConfigViper(object):
         
         if self._save_on_set:
             self.save()
+
+
+    def set_auto_save(self):
+        """Save configurations on every call to :meth:`set` so you do not have
+        to call :meth:`save` yourself. This is the default behavior.
+        """
+        self._save_on_set = True
+        self._stop_saving__save_on_set_flag = True
             
         
     def save(self):
         """Save configurations in JSON format, with indentation and 
-        sorted keys.
+        sorted keys. If :attr:`is_make_backups` is ``True`` a backup file will
+        be created right before the actual save. In case of failure, the
+        backup will be restored and is guaranteed that no exceptions will be
+        silenced on any failure.
         """
-        with open(self._confname, 'wb') as f:
-            f.write(json.dumps(self._config_map, sort_keys=True, indent=4))
+        # restore the save on set flag; this ensures that the default
+        # behavior is restored even if any error occurs;
+        self._save_on_set = self._stop_saving__save_on_set_flag
+
+        # make a backup before save (if required)...
+        if self._make_backups:
+            if os.path.exists(self._confname):
+                shutil.copyfile(self._confname, self._backupname)
+
+        # convert configurations dict to json;
+        data = json.dumps(self._config_map, sort_keys=True, indent=4)
+
+        try:
+            with open(self._confname, 'wb') as f:
+                f.write(data)
+
+        except:
+            # ouch! try to restore backup file (if there is one) and 
+            # honor keep_backups option
+            if self._make_backups and os.path.exists(self._backupname):
+                shutil.copyfile(self._backupname, self._confname)
+                if not self._keep_backups:
+                    os.unlink(self._backupname)
+
+            # and re-raise exception
+            raise
+
+        else:
+            # finally, keep or delete the backup file (if any)
+            if self._make_backups and (not self._keep_backups):
+                if os.path.exists(self._backupname):
+                    os.unlink(self._backupname)
+
+
+    def stop_saving(self):
+        """Stop saving on every :meth:`set`. This is particularly useful when
+        you need to set various configurations at once, to avoid the overburden
+        of save on every set. The next call to :meth:`save` will restore the
+        normal operation according to :attr:`is_auto_save`. The pattern is::
+
+            conf = ConfigViper()
+            conf.stop_saving()
+            conf.set('spam.ham.eggs', 'sausage')
+            conf.set('foo.bar', 'baz')
+            ..
+            conf.save()
+
+        """
+        self._stop_saving__save_on_set_flag = self._save_on_set
+        self._save_on_set = False
        
 
     def stabilize(self, values):
@@ -411,7 +493,21 @@ class ConfigViper(object):
         :attr:`pathname` and :attr:`filename`.
         """
         return self._confname
-        
+
+
+    @property
+    def backup_filename(self):
+        """The full path and file name for the backup. 
+        See :attr:`is_make_backups` and :meth:`save`.
+        """
+        return self._backupname
+    
+
+    @property
+    def path_separator(self):
+        """The config-path separator this instance is using."""
+        return self._path_separator
+
     
     @property
     def is_auto_save(self):
@@ -419,11 +515,24 @@ class ConfigViper(object):
         immediately before :meth:`set` method is called. This flag is also
         considered when :meth:`stabilize` method is called.
         """
-        return self._save_on_set
+        return self._save_on_set or self._stop_saving__save_on_set_flag
 
 
     @property
-    def path_separator(self):
-        """The config-path separator this instance is using."""
-        return self._path_separator
+    def is_make_backups(self):
+        """Indicates whether a backup should be made right before actual
+        save operation. The :meth:`save` method is capable of doing a backup
+        and restore from that backup if the actual save operation fails.
+        See :attr:`is_keep_backups`. The backup file will have the same name as
+        the original configuration file with ``.backup`` appended.
+        """
+        return self._make_backups
+
+
+    @property
+    def is_keep_backups(self):
+        """Indicate whether the backup file should be kept when the :meth:`save`
+        method completes, even if an error occurs. See :attr:`is_make_backups`.
+        """
+        return self._keep_backups
 
